import random
import bpy

from wg_utils import lazy_deselect_objects, opify_save_excursion

DEFAULT_PATENT_TEMPLATE = 'pat_msft'

# 
# patent objects go on layer 2, but we should operate with 1, 2, and 6 active
PATENT_OPERATE_LAYERS = [True, True, False, False, False, True] + [False] * 14
PATENT_LAYERS = [False, True] + [False] * 18


class AppearifyPatent(bpy.types.Operator):
    bl_idname = "patanim.appearify_patent"
    bl_label = "Appearify Patent"

    @opify_save_excursion
    def execute(self, context):
        bpy.context.scene.visible_layers = PATENT_OPERATE_LAYERS

        # Figure out which one we want (use a panel?)
        patent_template = bpy.context.scene.get(
            'patent_template', DEFAULT_PATENT_TEMPLATE)

        # Get the template objects
        template_text = bpy.data.objects[patent_template + '_txt']
        template_box = bpy.data.objects[patent_template + '_box']

        # Duplicate the appropriate objects
        ## Deselect all
        lazy_deselect_objects()

        ## Select the objects
        template_text.selected = True
        template_box.selected = True

        ## Duplicate them
        bpy.ops.object.duplicate()

        new_text = [
            obj for obj in bpy.context.selected_objects
            if obj.type == 'TEXT'][0]
        new_box = [
            obj for obj in bpy.context.selected_objects
            if obj.type == 'MESH'][0]

        # Move them to the cursor plus w/ template Z
        new_text.location = bpy.context.scene.cursor_location
        new_text.location[2] = template_text.location[2]
        new_box.location = bpy.context.scene.cursor_location
        new_box.location[2] = template_box.location[2]

        # Group them
        bpy.ops.group.create(name="pat_block")

        # Generate new text for the patent text
        new_text.data.body = "PAT\n#%s" % random.randrange(500000, 750000)

        # Animate 'em!
        orig_frame = bpy.context.scene.current_frame
        bpy.context.scene.set_frame(orig_frame + 6)

        # Last frame
        ## insert location and scaling keys
        bpy.ops.anim.keyframe_insert(type=-1, confirm_success=False)
        bpy.ops.anim.keyframe_insert(type=-3, confirm_success=False)

        # First frame
        bpy.context.scene.set_frame(orig_frame)
        bpy.ops.transform.resize(value=(0.0, 0.0, 0.0))
        bpy.ops.anim.keyframe_insert(type=-1, confirm_success=False)
        bpy.ops.anim.keyframe_insert(type=-3, confirm_success=False)

        # slightly mid-point frame
        ## move up slightly
        bpy.context.scene.set_frame(orig_frame + 4)
        new_text.location[1] = new_text.location[1] + .06
        new_box.location[1] = new_box.location[1] + .06
        bpy.ops.anim.keyframe_insert(type=-1, confirm_success=False)

        # Finally, set the layers...
        new_text.layers = PATENT_LAYERS
        new_box.layers = PATENT_LAYERS

        return {'FINISHED'}


bpy.types.register(AppearifyPatent)
